# This example requires the 'message_content' privileged intent to function.


import discord
from discord.ext import commands
from ductile import View, ViewObject
from ductile.controller import MessageableController
from ductile.pagination import Paginator
from ductile.ui import Button

PAGES = [
    "Page 1",
    "Page 2",
    "Page 3",
    "Page 4",
    "Page 5",
]


class Bot(commands.Bot):
    def __init__(self) -> None:
        super().__init__(command_prefix="!", intents=discord.Intents.all())

    async def on_ready(self) -> None:
        print(f"Logged in as {self.user}")  # noqa: T201
        print("Ready!")  # noqa: T201


class PaginationView(View):
    def __init__(self) -> None:
        super().__init__()
        # initialize Paginator with source data and configuration.
        self.page = Paginator(self, source=PAGES, config={"page_size": 2})

    def render(self) -> ViewObject:
        # you can define component callback inside render method.
        e = discord.Embed(title="Pagination").add_field(
            name="page content",
            # you can get current page chunk data.
            value="\n".join(self.page.data),
        )

        # Define UI using ViewObject
        return ViewObject(
            embeds=[e],
            components=[
                Button(
                    "<<",
                    # you can use at_first and at_last property: returns whether the current page is the first/last page.
                    style={"color": "grey", "disabled": self.page.at_first},
                    # go_first, go_previous, go_next, go_last methods: go to first/previous/next/last page.
                    # these will automatically call `View.sync`.
                    on_click=self.page.go_first,
                ),
                Button(
                    "<",
                    style={"color": "grey", "disabled": self.page.at_first},
                    on_click=self.page.go_previous,
                ),
                Button(
                    # you can get current page number and max page number.
                    f"{self.page.current_page}/{self.page.max_page}",
                    style={"color": "grey", "disabled": True},
                ),
                Button(
                    ">",
                    style={"color": "grey", "disabled": self.page.at_last},
                    on_click=self.page.go_next,
                ),
                Button(
                    ">>",
                    style={"color": "grey", "disabled": self.page.at_last},
                    on_click=self.page.go_last,
                ),
            ],
        )


bot = Bot()


@bot.command(name="page")
async def send_counter(ctx: commands.Context) -> None:
    # use controller to send and control View.
    controller = MessageableController(PaginationView(), messageable=ctx.channel)
    await controller.send()


bot.run("MY_COOL_BOT_TOKEN")



# This example requires the 'message_content' privileged intent to function.


import discord
from discord.ext import commands
from ductile import State, View, ViewObject
from ductile.controller import InteractionController, MessageableController
from ductile.ui import Button


class Bot(commands.Bot):
    def __init__(self) -> None:
        super().__init__(command_prefix="!", intents=discord.Intents.all())

    async def on_ready(self) -> None:
        print(f"Logged in as {self.user}")  # noqa: T201
        print("Ready!")  # noqa: T201


class ConfirmView(View):
    def __init__(self) -> None:
        super().__init__()
        self.approved = State[bool | None](None, self)
        self.description = State("Are you sure?", self)

    def render(self) -> ViewObject:
        async def handle_approve(interaction: discord.Interaction) -> None:
            await interaction.response.defer()
            self.approved.set_state(True)
            self.stop()

        async def handle_deny(interaction: discord.Interaction) -> None:
            await interaction.response.defer()
            self.approved.set_state(False)
            self.stop()

        return ViewObject(
            embeds=[discord.Embed(title="Confirm", description=self.description())],
            components=[
                Button("yes", style={"color": "green", "disabled": self.approved() is not None}, on_click=handle_approve),
                Button("no", style={"color": "red", "disabled": self.approved() is not None}, on_click=handle_deny),
            ],
        )

    async def on_timeout(self) -> None:
        self.description.set_state("Timed out.")
        self.approved.set_state(False)
        self.stop()


class SomeBreakingView(View):
    def __init__(self) -> None:
        super().__init__()
        # you can set state value type with generic type annotation.
        self.approved = State[bool | None](None, self)

    def render(self) -> ViewObject:
        e = discord.Embed(
            title="Some Breaking View",
            description="With great power comes great responsibility.",
        )
        e.add_field(name="Approved", value="Not selected yet" if self.approved() is None else self.approved())

        async def handle_execute(interaction: discord.Interaction) -> None:
            await interaction.response.send_message("Executing...")
            await interaction.edit_original_response(content="Executed!")

        async def wait_for_confirm(interaction: discord.Interaction) -> None:
            await interaction.response.defer()
            # you can set timeout in Controller constructor.
            # View.on_timeout will be called when the timeout expires.
            view = ConfirmView()
            controller = InteractionController(view, interaction=interaction, timeout=60, ephemeral=True)
            await controller.send()
            await controller.wait()
            self.approved.set_state(view.approved())

        return ViewObject(
            embeds=[e],
            components=[
                Button(
                    "execute",
                    style={"color": "green", "disabled": self.approved() is not True},
                    on_click=handle_execute,
                ),
                Button(
                    "confirm",
                    style={"color": "grey", "disabled": self.approved() is not None},
                    on_click=wait_for_confirm,
                ),
            ],
        )


bot = Bot()


@bot.command(name="breaking")
async def send_counter(ctx: commands.Context) -> None:
    controller = MessageableController(SomeBreakingView(), messageable=ctx.channel)
    await controller.send()
    await controller.wait()


bot.run("MY_COOL_BOT_TOKEN")



# This example requires the 'message_content' privileged intent to function.


from typing import Literal

import discord
from discord.ext import commands
from ductile import State, View, ViewObject
from ductile.controller import MessageableController
from ductile.ui import Button, Select


class Bot(commands.Bot):
    def __init__(self) -> None:
        super().__init__(command_prefix="!", intents=discord.Intents.all())

    async def on_ready(self) -> None:
        print(f"Logged in as {self.user}")  # noqa: T201
        print("Ready!")  # noqa: T201


class SelectView(View):
    def __init__(self) -> None:
        super().__init__()
        # you can set generic type annotation implicitly.
        self.current_page = State[Literal["1", "2", "3", "4"]]("1", self)
        self.disabled = State(False, self)  # noqa: FBT003

    def render(self) -> ViewObject:
        pages = {"1": "Page content 1", "2": "Page content 2", "3": "Page content 3", "4": "Page content 4"}

        # self.current_page() is equivalent to self.current_page.get_state()
        e = discord.Embed(title="Multi-Page Sample", description=pages[self.current_page()])

        async def handle_select(interaction: discord.Interaction, values: list[str]) -> None:
            await interaction.response.defer()

            new_value = values[0] if values[0] in pages else "1"  # new_value is "1" or "2" or "3" or "4"
            self.current_page.set_state(new_value)  # pyright: ignore[reportGeneralTypeIssues]

        async def stop(interaction: discord.Interaction) -> None:
            await interaction.response.defer()
            self.disabled.set_state(True)
            # stop the view. this will cause View.wait to return.
            self.stop()

        # Define UI using ViewObject
        return ViewObject(
            embeds=[e],
            components=[
                Select(
                    config={"min_values": 1, "max_values": 1},
                    style={"disabled": self.disabled(), "placeholder": "Select page to see."},
                    options=[
                        {
                            "label": f"page{page}",  # label: option label, displayed to user
                            "value": page,  # value: option value, returned to callback
                            "default": self.current_page() == page,
                        }
                        for page in pages
                    ],
                    on_select=handle_select,
                ),
                Button("stop", style={"disabled": self.disabled(), "color": "red"}, on_click=stop),
            ],
        )


bot = Bot()


@bot.command(name="page")
async def send_select(ctx: commands.Context) -> None:
    # you can set view timeout in Controller constructor.
    controller = MessageableController(SelectView(), messageable=ctx.channel, timeout=None)
    await controller.send()


bot.run("MY_COOL_BOT_TOKEN")



# This example requires the 'message_content' privileged intent to function.


import random

import discord
from discord.ext import commands
from ductile import State, View, ViewObject
from ductile.controller import MessageableController
from ductile.ui import Button


class Bot(commands.Bot):
    def __init__(self) -> None:
        super().__init__(command_prefix="!", intents=discord.Intents.all())

    async def on_ready(self) -> None:
        print(f"Logged in as {self.user}")  # noqa: T201
        print("Ready!")  # noqa: T201


class CounterView(View):
    def __init__(self) -> None:
        super().__init__()
        # state value type will be inferred from the initial value
        # self.count: State[int]
        self.count = State(0, self)

    def render(self) -> ViewObject:
        # you can define component callback inside render method.

        # use State.get_state to get the current value of the state.
        # also you can use State as callable to get the current value.
        # in this example, self.count() is equivalent to self.count.get_state()
        e = discord.Embed(title="Counter", description=f"Count: {self.count.get_state()}")

        async def handle_increment(interaction: discord.Interaction) -> None:
            # use defer if you only update State in callback.
            await interaction.response.defer()

            # if you pass callable to State.set_state, the callable will be called with the current value of the state.
            # in this example, x is the current value of the state.
            self.count.set_state(lambda x: x + 1)

        async def handle_decrement(interaction: discord.Interaction) -> None:
            await interaction.response.defer()
            self.count.set_state(lambda x: x - 1)

        async def handle_reset(interaction: discord.Interaction) -> None:
            await interaction.response.defer()
            # also you can pass new value to State.set_state.
            self.count.set_state(0)

        async def stop(interaction: discord.Interaction) -> None:
            await interaction.response.defer()
            # stop the view. this will cause View.wait to return.
            self.stop()

        # Define UI using ViewObject
        return ViewObject(
            embeds=[e],
            components=[
                # components are fully typed with TypedDict.
                # you can pass callback to Button.on_click.
                Button("+1", style={"color": "blurple", "row": 0}, on_click=handle_increment),
                Button("-1", style={"color": "blurple", "row": 0}, on_click=handle_decrement),
                Button(
                    "reset",
                    # you can set style with conditional expression.
                    style={"color": "grey", "row": 1, "disabled": self.count.get_state() == 0},
                    on_click=handle_reset,
                ),
                Button(
                    "random",
                    style={"color": "green", "row": 1},
                    # if you passed synchronous function to Button.on_click,
                    # library automatically calls `await interaction.response.defer()`.
                    on_click=lambda _: self.count.set_state(random.randint(0, 100)),  # noqa: S311
                ),
                Button("stop", style={"color": "red", "row": 1}, on_click=stop),
            ],
        )


bot = Bot()


@bot.command(name="counter")
async def send_counter(ctx: commands.Context) -> None:
    # use controller to send and control View.
    controller = MessageableController(CounterView(), messageable=ctx.channel)
    await controller.send()

    # await controller.wait returns a ViewResult, when View.stop is called.
    # you must call View.stop to get State out of the View.
    # ViewResult.timed_out is True if the View timed out.
    # ViewResult.states is a dict of State. the key is the name of instance attribute.
    timed_out, states = await controller.wait()
    await ctx.send(f"Timed out: {timed_out}\nCount: {states['count']}")


bot.run("MY_COOL_BOT_TOKEN")
